import QtQuick 1.0
import Qt3D 1.0
import Qt3D.Shapes 1.0

Viewport {
    id: main
    width: 864
    height: 480

    // the march of time
    property real t: 0
    property real speed: 86400/15*31
    property real tick: 0
    NumberAnimation {
        target: main; property: "tick"
        from: 0; to: 86400
        duration: 86400*1000
        running: true
    }
    // we do our own differentials here so the camera can _optionally_
    // follow the earth
    property real lastTick: 0
    property real oldCrystalTheta
    onTickChanged: {
        if (tick==lastTick) return
        oldCrystalTheta=crystal.theta
        var dt=(tick-lastTick)*speed
        t+=dt
        lastTick=tick
        if (controlPanel.trackEarth) {
            var dcth=crystal.theta-oldCrystalTheta
            camtheta+=dcth
        }
    }

    // viewport properties
    navigation: true // for demo
    blending: true

    // camera
    property real camtheta: 0
    property real camphi: 30 // up from ecliptic
    property real camr: 15 // units from sun
    property real camthetar: camtheta*Math.PI/180
    property real camphir: camphi*Math.PI/180
    camera: Camera {
        id: mainCamera
        eye: Qt.vector3d(
                 camr*Math.cos(camphir)*Math.sin(camthetar),
                 camr*Math.sin(camphir),
                 camr*Math.cos(camphir)*Math.cos(camthetar))
        upVector: "0.0,1.0,0.0"
        center: "0.0,0.0,0.0"
        fieldOfView: 45
        nearPlane: camr-2.2*controlPanel.maxSize
        farPlane: camr+2.2*controlPanel.maxSize
    }

    // light -- largely ignored
    light: Light {
        ambientColor: "#ffffff88"
        specularColor: "#000000"
        diffuseColor: "#ffffff44"
        position: "50,50,50"
    }

    Item3D {
        id: crystal
        property real theta: main.t*360/86400/365.25
        sortChildren: "BackToFront"
        transform: [
            Rotation3D {
                angle: crystal.theta
                axis: "0.0,1.0,0.0"
            }
        ]
        Repeater {
            model: 7
            Repeater {
                property real irx: index-3
                model: 7
                Repeater {
                    property real iry: index-3
                    model: 7
                    UnitCell {
                        ix: irx
                        iy: iry
                        iz: index-3
                    }
                }
            }
        }
    }

    ControlPanel {
        id: controlPanel
    }
}
